@@ -46,9 +46,18 @@ def test_loader_with_valid_svn_externals(
         repo_url,
         "Create repository structure.",
         [
-            CommitChange(change_type=CommitChangeType.AddOrUpdate, path="branches/",),
-            CommitChange(change_type=CommitChangeType.AddOrUpdate, path="tags/",),
-            CommitChange(change_type=CommitChangeType.AddOrUpdate, path="trunk/",),
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="branches/",
+            ),
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="tags/",
+            ),
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="trunk/",
+            ),
             CommitChange(
                 change_type=CommitChangeType.AddOrUpdate,
                 path="trunk/bar.sh",
@@ -84,7 +93,10 @@ def test_loader_with_valid_svn_externals(
     loader = SvnLoader(swh_storage, repo_url, temp_directory=tmp_path, check_revision=1)
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -105,21 +117,33 @@ def test_loader_with_valid_svn_externals(
     loader = SvnLoader(swh_storage, repo_url, temp_directory=tmp_path, check_revision=1)
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
 
-def test_loader_with_invalid_svn_externals(swh_storage, repo_url, tmp_path):
+def test_loader_with_invalid_svn_externals(swh_storage, repo_url, tmp_path, mocker):
 
     # first commit
     add_commit(
         repo_url,
         "Create repository structure.",
         [
-            CommitChange(change_type=CommitChangeType.AddOrUpdate, path="branches/",),
-            CommitChange(change_type=CommitChangeType.AddOrUpdate, path="tags/",),
-            CommitChange(change_type=CommitChangeType.AddOrUpdate, path="trunk/",),
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="branches/",
+            ),
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="tags/",
+            ),
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="trunk/",
+            ),
         ],
     )
 
@@ -147,7 +171,10 @@ def test_loader_with_invalid_svn_externals(swh_storage, repo_url, tmp_path):
     loader = SvnLoader(swh_storage, repo_url, temp_directory=tmp_path, check_revision=1)
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -222,7 +249,10 @@ def test_loader_with_valid_externals_modification(
     loader = SvnLoader(swh_storage, repo_url, temp_directory=tmp_path, check_revision=1)
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -289,7 +319,10 @@ def test_loader_with_valid_externals_and_versioned_path(
     loader = SvnLoader(swh_storage, repo_url, temp_directory=tmp_path, check_revision=1)
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -331,7 +364,10 @@ def test_loader_with_invalid_externals_and_versioned_path(
     loader = SvnLoader(swh_storage, repo_url, temp_directory=tmp_path, check_revision=1)
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -388,7 +424,10 @@ def test_loader_set_externals_then_remove_and_add_as_local(
     loader = SvnLoader(swh_storage, repo_url, temp_directory=tmp_path, check_revision=1)
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -426,7 +465,10 @@ def test_loader_set_invalid_externals_then_remove(swh_storage, repo_url, tmp_pat
     loader = SvnLoader(swh_storage, repo_url, temp_directory=tmp_path, check_revision=1)
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -493,7 +535,10 @@ def test_loader_set_externals_with_versioned_file_overlap(
     loader = SvnLoader(swh_storage, repo_url, temp_directory=tmp_path, check_revision=1)
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -544,7 +589,10 @@ def test_dump_loader_relative_externals_detection(
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
     assert loader.svnrepo.has_relative_externals
@@ -566,7 +614,10 @@ def test_dump_loader_relative_externals_detection(
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
     assert not loader.svnrepo.has_relative_externals
@@ -593,8 +644,14 @@ def test_loader_externals_cache(swh_storage, repo_url, external_repo_url, tmp_pa
         repo_url,
         "Create repository structure.",
         [
-            CommitChange(change_type=CommitChangeType.AddOrUpdate, path="project1/",),
-            CommitChange(change_type=CommitChangeType.AddOrUpdate, path="project2/",),
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="project1/",
+            ),
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="project2/",
+            ),
         ],
     )
 
@@ -624,7 +681,10 @@ def test_loader_externals_cache(swh_storage, repo_url, external_repo_url, tmp_pa
     loader = SvnLoader(swh_storage, repo_url, temp_directory=tmp_path, check_revision=1)
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -685,15 +745,26 @@ def test_loader_remove_versioned_path_with_external_overlap(
     add_commit(
         repo_url,
         "Remove trunk/project/ versioned path",
-        [CommitChange(change_type=CommitChangeType.Delete, path="trunk/project/",),],
+        [
+            CommitChange(
+                change_type=CommitChangeType.Delete,
+                path="trunk/project/",
+            ),
+        ],
     )
 
     loader = SvnLoader(
-        swh_storage, repo_url, temp_directory=tmp_path, check_revision=1,
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -718,7 +789,12 @@ def test_loader_export_external_path_using_peg_rev(
     add_commit(
         external_repo_url,
         "Remove previously added file",
-        [CommitChange(change_type=CommitChangeType.Delete, path="code/foo.sh",),],
+        [
+            CommitChange(
+                change_type=CommitChangeType.Delete,
+                path="code/foo.sh",
+            ),
+        ],
     )
 
     # third commit on external
@@ -738,7 +814,12 @@ def test_loader_export_external_path_using_peg_rev(
     add_commit(
         repo_url,
         "Add trunk dir",
-        [CommitChange(change_type=CommitChangeType.AddOrUpdate, path="trunk/",),],
+        [
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="trunk/",
+            ),
+        ],
     )
 
     # second commit
@@ -776,11 +857,17 @@ def test_loader_export_external_path_using_peg_rev(
     )
 
     loader = SvnLoader(
-        swh_storage, repo_url, temp_directory=tmp_path, check_revision=1,
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -853,11 +940,17 @@ def test_loader_remove_external_overlapping_versioned_path(
     )
 
     loader = SvnLoader(
-        swh_storage, repo_url, temp_directory=tmp_path, check_revision=1,
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -914,11 +1007,17 @@ def test_loader_modify_external_same_path(
     )
 
     loader = SvnLoader(
-        swh_storage, repo_url, temp_directory=tmp_path, check_revision=1,
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -972,22 +1071,34 @@ def test_loader_with_recursive_external(
 
     # first load
     loader = SvnLoader(
-        swh_storage, repo_url, temp_directory=tmp_path, check_revision=1,
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
     assert loader.svnrepo.has_recursive_externals
 
     # second load on stale repo
     loader = SvnLoader(
-        swh_storage, repo_url, temp_directory=tmp_path, check_revision=1,
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
     )
     assert loader.load() == {"status": "uneventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
     assert loader.svnrepo.has_recursive_externals
@@ -1009,11 +1120,17 @@ def test_loader_with_recursive_external(
 
     # third load
     loader = SvnLoader(
-        swh_storage, repo_url, temp_directory=tmp_path, check_revision=1,
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
     assert not loader.svnrepo.has_recursive_externals
@@ -1066,11 +1183,17 @@ def test_loader_externals_with_same_target(
     )
 
     loader = SvnLoader(
-        swh_storage, repo_url, temp_directory=tmp_path, check_revision=1,
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -1119,11 +1242,17 @@ def test_loader_external_in_versioned_path(
     )
 
     loader = SvnLoader(
-        swh_storage, repo_url, temp_directory=tmp_path, check_revision=1,
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
 
@@ -1263,10 +1392,176 @@ def test_loader_externals_add_remove_readd_on_subpath(
     )
 
     loader = SvnLoader(
-        swh_storage, repo_url, temp_directory=tmp_path, check_revision=1,
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
     )
     assert loader.load() == {"status": "eventful"}
     assert_last_visit_matches(
-        loader.storage, repo_url, status="full", type="svn",
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
+    )
+    check_snapshot(loader.snapshot, loader.storage)
+
+
+def test_loader_directory_symlink_in_external(
+    swh_storage, repo_url, external_repo_url, tmp_path
+):
+    # first commit on external
+    add_commit(
+        external_repo_url,
+        "Create dirs in an external repository",
+        [
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="src/apps/",
+            ),
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="src/deps/",
+            ),
+        ],
+    )
+
+    # second commit on external
+    add_commit(
+        external_repo_url,
+        "Add symlink to src/deps in src/apps directory",
+        [
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="src/apps/deps",
+                data=b"link ../deps",
+                properties={"svn:special": "*"},
+            ),
+        ],
+    )
+
+    # first commit
+    add_commit(
+        repo_url,
+        "Add deps dir",
+        [CommitChange(change_type=CommitChangeType.AddOrUpdate, path="deps/")],
+    )
+
+    # second commit
+    add_commit(
+        repo_url,
+        "Set external to deps folder",
+        [
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="deps/",
+                properties={"svn:externals": (f"{external_repo_url} external")},
+            ),
+        ],
+    )
+
+    loader = SvnLoader(
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
+    )
+    assert loader.load() == {"status": "eventful"}
+    assert_last_visit_matches(
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
+    )
+    check_snapshot(loader.snapshot, loader.storage)
+
+
+def test_loader_with_externals_parsing_error(
+    swh_storage, repo_url, external_repo_url, tmp_path
+):
+    # first commit on external
+    add_commit(
+        external_repo_url,
+        "Create code directory",
+        [
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="code/",
+            ),
+        ],
+    )
+
+    # second commit on external
+    add_commit(
+        external_repo_url,
+        "Create code/foo.sh file",
+        [
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="code/foo.sh",
+                properties={"svn:executable": "*"},
+                data=b"#!/bin/bash\necho foo",
+            ),
+        ],
+    )
+
+    # first commit
+    add_commit(
+        repo_url,
+        "Create trunk directory.",
+        [
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="trunk/",
+            ),
+        ],
+    )
+
+    # second commit
+    add_commit(
+        repo_url,
+        "Set external on trunk directory that will result in a parsing error.",
+        [
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="trunk/",
+                properties={
+                    "svn:externals": (
+                        f"-r2{svn_urljoin(external_repo_url, 'code/foo.sh')} foo.sh"
+                    )
+                },
+            ),
+        ],
+    )
+
+    # third commit
+    add_commit(
+        repo_url,
+        "Fix external definition on trunk directory.",
+        [
+            CommitChange(
+                change_type=CommitChangeType.AddOrUpdate,
+                path="trunk/",
+                properties={
+                    "svn:externals": (
+                        f"-r2 {svn_urljoin(external_repo_url, 'code/foo.sh')} foo.sh"
+                    )
+                },
+            ),
+        ],
+    )
+
+    loader = SvnLoader(
+        swh_storage,
+        repo_url,
+        temp_directory=tmp_path,
+        check_revision=1,
+    )
+    assert loader.load() == {"status": "eventful"}
+    assert_last_visit_matches(
+        loader.storage,
+        repo_url,
+        status="full",
+        type="svn",
     )
     check_snapshot(loader.snapshot, loader.storage)
